/***************************************************************************
 *
 *   ::::_audioeditor_::::
 *   a simple multitrack audio editor
 *
 *   mainwindow
 *   header file
 *
 *   begin                : Jan 2006
 *   copyright            : (C) 2006 by Georg Holzmann
 *   email                : grh@mur.at
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

#ifndef MAINWINDOW_H__
#define MAINWINDOW_H__

#include <QMainWindow>
#include "globaldefines.h"

class QAction;
class QMenu;
class AudioCtl;
class TrackManager;

/*!
 * \class MainWindow
 *
 * The main window of the application.
 * Handles Tool-, Menu-, Statusbar, read/write application settings, etc.
 */
class MainWindow : public QMainWindow
{
    Q_OBJECT

  public:
    MainWindow();
    virtual ~MainWindow();

  protected:
    void closeEvent(QCloseEvent *event);

  private slots:
    void newFile();
    void open();
    bool save();
    bool saveAs();
    void about();
    void documentWasModified();
    void playSlot();
    void stopSlot();

  private:
    
    /*! 
     * \brief read application settigs
     *
     * This uses QSettings, a platform-independent application
     * settings framework.
     * On Windows, it uses the (in)famous Windows registry; on Mac OS X,
     * it uses the native XML-based CFPreferences API; on Unix/X11,
     * it uses text files.
     * For the detailed location of the file see qt4 documentation.
     */
    void readSettings();
    
    /*! 
     * \brief write application settigs
     *
     * This uses QSettings, a platform-independent application
     * settings framework.
     * On Windows, it uses the (in)famous Windows registry; on Mac OS X,
     * it uses the native XML-based CFPreferences API; on Unix/X11,
     * it uses text files.
     * For the detailed location of the file see qt4 documentation.
     */
    void writeSettings();

    /*!
     * asks the user if he wants to save a changed project
     * @return true if closing is allowed
     */
    bool maybeSave();
    
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    
    void loadFile(const QString &fileName);
    bool saveFile(const QString &fileName);
    void setCurrentFile(const QString &fileName);
    QString strippedName(const QString &fullFileName);

    /*! the widget with all the tracks */
    TrackManager *m_trackmanager;
    
    /*! the audio controller */
    AudioCtl *m_audioctl;
    
    /*! current project filename */
    QString m_curFile;

    QMenu *fileMenu;
    QMenu *editMenu;
    QMenu *playbackMenu;
    QMenu *helpMenu;
    QToolBar *fileToolBar;
    QToolBar *editToolBar;
    QToolBar *playbackToolBar;
    QAction *newAct;
    QAction *openAct;
    QAction *saveAct;
    QAction *saveAsAct;
    QAction *exitAct;
    QAction *cutAct;
    QAction *copyAct;
    QAction *pasteAct;
    QAction *playAct;
    QAction *stopAct;
    QAction *aboutAct;
    
    /*! helper for play/pause/stop logic */
    bool m_stopped;
};

#endif // MAINWINDOW_H__
