/***************************************************************************
 *
 *   ::::_audioeditor_::::
 *   a simple multitrack audio editor
 *
 *   soundfile
 *   source file
 *
 *   begin                : Jan 2006
 *   copyright            : (C) 2006 by Georg Holzmann
 *   email                : grh@mur.at
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

#include "soundfile.h"
#include "globaldefines.h"
#include <QMessageBox>
#include "audioctl.h"

Soundfile::Soundfile() : m_sndfile(NULL)
{
  m_audioctl = new AudioCtl;
  connect(m_audioctl, SIGNAL(positionChanged(Position new_position)),
          this, SLOT(seek(Position new_pos)));
  
  // TESTING:
  open("../test.wav");
}

Soundfile::~Soundfile()
{
  close();
  
  delete m_audioctl;
}

bool Soundfile::open(const QString &fileName)
{
  m_sndfile_lock.lockForRead();
  if(m_sndfile)
    close();
  m_sndfile_lock.unlock();
  
  // TODO: schauen, ob i fileName.data() brauche !?)
  m_sndfile_lock.lockForWrite();
  m_info_lock.lockForWrite();
  m_sndfile = sf_open(fileName.toAscii(), SFM_READ, &m_info);
  m_info_lock.unlock();
  m_sndfile_lock.unlock();
  
  m_sndfile_lock.lockForRead();
  if(!m_sndfile)
  {
    QMessageBox::warning(0, tr("Soundfile Loading Error"), 
                          tr("Cannot open soundfile %1 - please check the path and filename!")
                              .arg(fileName) );
    return false;
  }
  m_sndfile_lock.unlock();
  
  m_info_lock.lockForRead();
  if(m_info.channels > 2)
  {
    QMessageBox::warning(0, tr("Soundfile Loading Error"), 
                         tr("%1: this file has too much channels (only mono and stereo allowed) !")
                             .arg(fileName) );
    close();
    return false;
  }
  if(m_info.samplerate != DEFAULT_SAMPLERATE)
  {
    QMessageBox::warning(0, tr("Soundfile Loading Error"), 
                         tr("%1: this file has a wrong samplerate - please convert it to 44100 !")
                             .arg(fileName) );
    close();
    return false;
  }
  if(!m_info.seekable)
  {
    QMessageBox::warning(0, tr("Soundfile Loading Error"), 
                         tr("TODO: %1: not seekable ! -> swap file implementieren !!!")
                             .arg(fileName) );
    close();
    return false;
  }
  m_info_lock.unlock();
  
  return true;
}

void Soundfile::close()
{
  m_sndfile_lock.lockForRead();
  if(m_sndfile)
    Q_ASSERT( !sf_close(m_sndfile) );
  m_sndfile_lock.unlock();
  
  m_sndfile_lock.lockForWrite();
  m_sndfile = NULL;
  m_sndfile_lock.unlock();
}

int Soundfile::process(unsigned long size, float *output)
{
  m_sndfile_lock.lockForRead();
  
  if(!m_sndfile)
    return -1;
  
  m_info_lock.lockForRead();
  int channels = m_info.channels;
  m_info_lock.unlock();
  
  float samples[2];
  sf_count_t read_samples = 0;
  
  // STEREO:
  if(channels==2)
  {
    for (unsigned int i = 0; i < size; ++i)
    {
      read_samples = sf_read_float(m_sndfile, samples, 2);
      
      // add sample data
      *output += samples[0]; *output++;
      *output += samples[1]; *output++;
    }
  }
  
  // MONO:
  if(channels==1)
  {
    for (unsigned int i = 0; i < size; ++i)
    {
      read_samples = sf_read_float(m_sndfile, samples, 1);
      
      // add sample data
      *output += samples[0]; *output++;
      *output += samples[0]; *output++;
    }
  }
  
  m_sndfile_lock.unlock();
  
  if(!read_samples)
    return -1;
  
  // TESTING: noise
  /*for( unsigned int i=0; i<size; i++ )
  {
    *output++ = ((float)(rand() % 99999)*2)/99999 - 1;
    *output++ = ((float)(rand() % 99999)*2)/99999 - 1;
  }
  */
  
  return 0;
}

void Soundfile::seek(Position new_pos)
{
  // TODO:
  // wenn mehrere tracks versch. offsets -> offset muss da halt
  //   dazugerechnet werden immer !!!
  //   -> auch play logic allgemein muss dann anders sein, weils
  //      ja dann net von Anfang an spielt etc.
  
  m_sndfile_lock.lockForWrite();
  
  int frames = sf_seek(m_sndfile, new_pos, SEEK_SET);
  
  // if seek beyond the end, set position to the end
  // TODO: testen ob das funktioniert !!!
  if(frames==-1)
    sf_seek(m_sndfile, 0, SEEK_END);
  
  m_sndfile_lock.unlock();
}
