/***************************************************************************
 *
 *   ::::_audioeditor_::::
 *   a simple multitrack audio editor
 *
 *   soundfile
 *   header file
 *
 *   begin                : Jan 2006
 *   copyright            : (C) 2006 by Georg Holzmann
 *   email                : grh@mur.at
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

#ifndef SOUNDFILE_H__
#define SOUNDFILE_H__

#include <QObject>
#include "sndfile.h"
#include "globaldefines.h"
#include <QReadWriteLock>

class AudioCtl;

/*!
 * \class Soundfile
 *
 * class for managing soundfileIO for a single track
 * it is using the library libsndfile - see http://www.mega-nerd.com/libsndfile/
 */
class Soundfile : public QObject
{
  Q_OBJECT
  
  public:
    Soundfile();
    virtual ~Soundfile();
    
    /*! 
     * opens a mono or stereo soundfile
     * @param fileName path to the file
     * @return true if successful
     */
    bool open(const QString &fileName);
    
    /*! close the soundfile if loaded */
    void close();
    
    /*!
     * adds the audiodata of this track to the buffer output
     * \note everything here must be threadsafe !!!
     * @param size nr. of samples to write (bufsize)
     * @param output output buffer: always stereo !!! (see AudioCtl
     *               how to write into this buffer !)
     * @return -1 if file is over, otherwise 0
     * 
     * \note there are a number of locks here in this function,
     *       which should be avoided in future !!!
     *       Also the file access etc. should be done in
     *       a separate thread !
     */
    int process(unsigned long size, float *output);
    
  public slots:
    
    /*!
     * seek in the sound file
     * @param new_pos seek in samples from the beginning
     */
    void seek(Position new_pos);
    
  protected:
    
    /*! the soundfile pointer and it's lock */
    SNDFILE* m_sndfile;
    QReadWriteLock m_sndfile_lock;
    
    /*! info about the soundfile and it's lock */
    SF_INFO m_info;
    QReadWriteLock m_info_lock;
    
    /*! the audio controller */
    AudioCtl *m_audioctl;
};

#endif // SOUNDFILE_H__
